/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//
// Created by webpigeon on 24/04/22.
//

#ifndef FGGL_PHYS_TYPES_HPP
#define FGGL_PHYS_TYPES_HPP

#include "fggl/phys/callbacks.hpp"

namespace fggl::phys {
	constexpr float MASS_STATIC = 0.0F;
	constexpr float MASS_DEFAULT = 1.0F;
	constexpr math::vec3 UNIT_EXTENTS{0.5F, 0.5F, 0.5F};

	enum class ShapeType {
			UNSET,
			BOX,
			SPHERE
	};

	struct PhyShape {
		math::vec3 offset = math::VEC3_ZERO;
		const ShapeType type;

		inline PhyShape(ShapeType typ) : type(typ) {}
	};

	struct Box : public PhyShape {
		math::vec3 extents;
		explicit inline Box(math::vec3 ext) : PhyShape(ShapeType::BOX), extents(ext) {}
	};

	struct Sphere : public PhyShape {
		float radius = 1.0F;
		explicit inline Sphere(float rad) : PhyShape(ShapeType::SPHERE), radius(rad) {}
	};

	enum class BodyType {
			STATIC, KINEMATIC, DYNAMIC
	};

	struct RigidBody {
		constexpr static const char* name = "phys::Body";
		float mass = MASS_DEFAULT;
		PhyShape* shape = nullptr;
		BodyType type = BodyType::DYNAMIC;

		[[nodiscard]]
		inline bool isStatic() const {
			return mass == MASS_STATIC;
		}
	};

	struct Dynamics {
		constexpr static const char* name = "phys::Dynamics";
		math::vec3 force = math::VEC3_ZERO;
	};

	class PhysicsEngine {
		public:
			PhysicsEngine() = default;
			virtual ~PhysicsEngine() = default;

			// no copy and no move
			PhysicsEngine(PhysicsEngine&) = delete;
			PhysicsEngine(PhysicsEngine&&) = delete;
			PhysicsEngine& operator=(PhysicsEngine&) = delete;
			PhysicsEngine& operator=(PhysicsEngine&&) = delete;

			virtual void step() = 0;
	};

} // namespace fggl::phys

#endif //FGGL_PHYS_TYPES_HPP
